@@ -1,5 +1,6 @@
 from django.contrib.auth import get_user_model
 from django.core.exceptions import ObjectDoesNotExist
+from django.test import tag
 from django.urls.base import reverse
 from django_webtest import WebTest
 from edc_appointment.models import Appointment
@@ -8,6 +9,7 @@ from edc_facility.import_holidays import import_holidays
 from edc_lab.models.panel import Panel
 from edc_lab.site_labs import site_labs
 from edc_reference.site_reference import site_reference_configs
+from edc_test_utils.get_webtest_form import get_webtest_form
 from edc_utils.date import get_utcnow
 from edc_visit_schedule.site_visit_schedules import site_visit_schedules
 from edc_visit_tracking.constants import SCHEDULED
@@ -67,7 +69,7 @@ class ModelAdminSiteTest(WebTest):
     def login(self):
         form = self.app.get(reverse("admin:index")).maybe_follow().form
         form["username"] = self.user.username
-        form["password"] = "pass"
+        form["password"] = "pass"  # nosec B105
         return form.submit()
 
     def test_redirect_next(self):
@@ -93,8 +95,9 @@ class ModelAdminSiteTest(WebTest):
         url = reverse(f"admin:edc_model_admin_{model}_add") + "?" + query_string
 
         response = self.app.get(url, user=self.user)
-        response.form["subject_identifier"] = self.subject_identifier
-        response = response.form.submit(name="_save").follow()
+        form = get_webtest_form(response)
+        form["subject_identifier"] = self.subject_identifier
+        response = form.submit(name="_save").follow()
 
         self.assertIn("You are at the subject dashboard", response)
         self.assertIn(self.subject_identifier, response)
@@ -121,7 +124,8 @@ class ModelAdminSiteTest(WebTest):
 
         # oops, cancel
         response = self.app.get(url, user=self.user)
-        response = response.form.submit(name="_cancel").follow()
+        form = get_webtest_form(response)
+        response = form.submit(name="_cancel").follow()
         self.assertIn("You are at the subject dashboard", response)
 
         # add CRF Two
@@ -132,9 +136,10 @@ class ModelAdminSiteTest(WebTest):
             "report_datetime_0": get_utcnow().strftime("%Y-%m-%d"),
             "report_datetime_1": "00:00:00",
         }
+        form = get_webtest_form(response)
         for key, value in form_data.items():
-            response.form[key] = value
-        response = response.form.submit(name="_savenext").follow()
+            form[key] = value
+        response = form.submit(name="_savenext").follow()
 
         # goes directly to CRF Three, add CRF Three
         self.assertIn("Add crf three", response)
@@ -143,9 +148,10 @@ class ModelAdminSiteTest(WebTest):
             "report_datetime_0": get_utcnow().strftime("%Y-%m-%d"),
             "report_datetime_1": "00:00:00",
         }
+        form = get_webtest_form(response)
         for key, value in form_data.items():
-            response.form[key] = value
-        response = response.form.submit(name="_savenext").follow()
+            form[key] = value
+        response = form.submit(name="_savenext").follow()
 
         # goes to dashboard
         self.assertIn("You are at the subject dashboard", response)
@@ -160,7 +166,8 @@ class ModelAdminSiteTest(WebTest):
         )
 
         response = self.app.get(url, user=self.user)
-        response = response.form.submit(name="_cancel").follow()
+        form = get_webtest_form(response)
+        response = form.submit(name="_cancel").follow()
         self.assertIn("You are at the subject dashboard", response)
 
         response = self.app.get(url, user=self.user)
@@ -170,9 +177,10 @@ class ModelAdminSiteTest(WebTest):
             "report_datetime_0": get_utcnow().strftime("%Y-%m-%d"),
             "report_datetime_1": "00:00:00",
         }
+        form = get_webtest_form(response)
         for key, value in form_data.items():
-            response.form[key] = value
-        response = response.form.submit(name="_savenext").follow()
+            form[key] = value
+        response = form.submit(name="_savenext").follow()
 
         self.assertIn("You are at the subject dashboard", response)
         self.assertIn(self.subject_identifier, response)
@@ -203,7 +211,8 @@ class ModelAdminSiteTest(WebTest):
         add_url = reverse(f"admin:edc_model_admin_{model}_add")
         url = add_url + f"?{query_string}&panel={str(panel_one.id)}"
         response = self.app.get(url, user=self.user)
-        response = response.form.submit(name="_cancel").follow()
+        form = get_webtest_form(response)
+        response = form.submit(name="_cancel").follow()
         self.assertIn("You are at the subject dashboard", response)
 
         dte = get_utcnow()
@@ -223,10 +232,11 @@ class ModelAdminSiteTest(WebTest):
         response = self.app.get(url, user=self.user)
         self.assertIn("Add requisition", response)
         self.assertIn(f'value="{str(panel_one.id)}"', response)
+        form = get_webtest_form(response)
         for key, value in form_data.items():
-            response.form[key] = value
-        response.form["requisition_identifier"] = "ABCDE0001"
-        response = response.form.submit().follow()
+            form[key] = value
+        form["requisition_identifier"] = "ABCDE0001"
+        response = form.submit().follow()
         self.assertIn("You are at the subject dashboard", response)
         Requisition.objects.all().delete()
 
@@ -237,16 +247,18 @@ class ModelAdminSiteTest(WebTest):
         self.assertIn("Add requisition", response)
         self.assertIn(f'value="{str(panel_one.id)}"', response)
         self.assertIn("_savenext", response)
+        form = get_webtest_form(response)
         for key, value in form_data.items():
-            response.form[key] = value
-        response.form["requisition_identifier"] = "ABCDE0001"
-        response = response.form.submit(name="_savenext").follow()
+            form[key] = value
+        form["requisition_identifier"] = "ABCDE0001"
+        response = form.submit(name="_savenext").follow()
         self.assertIn("Add requisition", response)
         self.assertIn(f'value="{str(panel_two.id)}"', response)
+        form = get_webtest_form(response)
         for key, value in form_data.items():
-            response.form[key] = value
-        response.form["requisition_identifier"] = "ABCDE0002"
-        response = response.form.submit(name="_savenext").follow()
+            form[key] = value
+        form["requisition_identifier"] = "ABCDE0002"
+        response = form.submit(name="_savenext").follow()
         self.assertIn("You are at the subject dashboard", response)
         self.assertIn(self.subject_identifier, response)
 
@@ -261,7 +273,8 @@ class ModelAdminSiteTest(WebTest):
         self.assertIn("Change requisition", response)
         self.assertIn("ABCDE0001", response)
         self.assertIn(f'{str(panel_one.id)}" selected>One</option>', response)
-        response = response.form.submit(name="_savenext").follow()
+        form = get_webtest_form(response)
+        response = form.submit(name="_savenext").follow()
 
         # self.assertIn("Change requisition", response)
         # self.assertIn("ABCDE0002", response)
@@ -295,9 +308,10 @@ class ModelAdminSiteTest(WebTest):
             "report_datetime_1": "00:00:00",
         }
         response = self.app.get(url, user=self.user)
+        form = get_webtest_form(response)
         for key, value in form_data.items():
-            response.form[key] = value
-        response.form.submit(name="_save").follow()
+            form[key] = value
+        form.submit(name="_save").follow()
 
         # delete
         crffour = CrfFour.objects.all()[0]
@@ -311,12 +325,14 @@ class ModelAdminSiteTest(WebTest):
         response = response.click(href=delete_url)
 
         # submit confirmation page
-        response = response.form.submit().follow()
+        form = get_webtest_form(response)
+        response = form.submit().follow()
 
         # redirects to the dashboard
         self.assertIn("You are at the subject dashboard", response)
         self.assertRaises(ObjectDoesNotExist, CrfFour.objects.get, id=crffour.id)
 
+    @tag("1")
     def test_redirect_on_delete_with_url_name_from_admin(self):
         self.login()
 
@@ -329,7 +345,8 @@ class ModelAdminSiteTest(WebTest):
         response = self.app.get(url, user=self.user)
         delete_url = reverse(f"admin:edc_model_admin_{model}_delete", args=(crffive.id,))
         response = response.click(href=delete_url)
-        response = response.form.submit().follow()
+        form = get_webtest_form(response)
+        response = form.submit().follow()
         self.assertIn("You are at Dashboard Two", response)
         self.assertRaises(ObjectDoesNotExist, CrfFive.objects.get, id=crffive.id)
 
@@ -345,6 +362,7 @@ class ModelAdminSiteTest(WebTest):
         response = self.app.get(url, user=self.user)
         delete_url = reverse(f"admin:edc_model_admin_{model}_delete", args=(crfsix.id,))
         response = response.click(href=delete_url)
-        response = response.form.submit().follow()
+        form = get_webtest_form(response)
+        response = form.submit().follow()
         self.assertRaises(ObjectDoesNotExist, CrfSix.objects.get, id=crfsix.id)
         self.assertIn("changelist", response)
